import os
import unittest
from pathlib import Path

import numpy
import obspy.core
from numpy.testing import assert_array_equal

from sohstationviewer.model.reftek_data.reftek_reader.core import (
    DiscontinuousTrace,
    Reftek130,
)
from sohstationviewer.model.reftek_data.reftek_reader.header import NotRT130FileError


class TestDiscontinuousTrace(unittest.TestCase):
    def setUp(self) -> None:
        data = numpy.arange(1024)
        stub_stats = obspy.core.Stats()
        times = numpy.arange(1024)
        self.trace = DiscontinuousTrace(data, stub_stats, times=times)

    def test_times_argument_is_stored(self):
        self.assertTrue(hasattr(self.trace, '_times'))

    def test_times_utcdatetime(self):
        with self.assertRaises(NotImplementedError):
            self.trace.times('utcdatetime')

    def test_times_matplotlib(self):
        with self.assertRaises(NotImplementedError):
            self.trace.times('matplotlib')

    def test_times_relative(self):
        with self.subTest('test_relative_to_start_time'):
            # The default start time of a trace is 0 anyhow, but we write that
            # down explicitly for clarity.
            self.trace.stats.starttime = obspy.core.UTCDateTime(0)
            expected = numpy.arange(1024)
            assert_array_equal(self.trace.times('relative'), expected)

        with self.subTest('test_relative_to_given_reftime'):
            reftime = obspy.core.UTCDateTime(0)
            expected = numpy.arange(1024)
            assert_array_equal(self.trace.times('relative', reftime),
                               expected)

            reftime = obspy.core.UTCDateTime(1024)
            expected = numpy.arange(-1024, 0)
            assert_array_equal(self.trace.times('relative', reftime),
                               expected)

            reftime = obspy.core.UTCDateTime(-1024)
            expected = numpy.arange(1024, 2048)
            assert_array_equal(self.trace.times('relative', reftime),
                               expected)

    def test_times_timestamp(self):
        expected = numpy.arange(1024)
        assert_array_equal(self.trace.times('timestamp'), expected)


class TestReftek130FromFile(unittest.TestCase):
    def setUp(self) -> None:
        self.TEST_DATA_DIR = Path(os.getcwd()).joinpath('tests/test_data')
        self.rt130_dir = self.TEST_DATA_DIR.joinpath(
            'RT130-sample/2017149.92EB/2017150/92EB'
        )

    def test_rt130_file(self):
        file = self.rt130_dir.joinpath('0/000000000_00000000')
        rt130 = Reftek130.from_file(file)
        self.assertIsInstance(rt130, Reftek130)

    def test_rt130_soh_file(self):
        file = self.rt130_dir.joinpath('0/000000000_00000000')
        rt130 = Reftek130.from_file(file)
        # The most common SOH packet type looks to be SH, so we use that as
        # the default.
        self.assertIn(b'SH', rt130._data['packet_type'])

    def test_rt130_raw_data_file(self):
        file = self.rt130_dir.joinpath('1/000000015_0036EE80')
        rt130 = Reftek130.from_file(file)
        assert_array_equal(
            numpy.unique(numpy.sort(rt130._data['packet_type'])),
            numpy.sort([b'EH', b'DT', b'ET'])
        )

    def test_non_rt130_file(self):
        with self.subTest('test_file_exist'):
            test_file = self.TEST_DATA_DIR.joinpath(
                'Q330-sample/day_vols_AX08/AX08.XA..HHE.2021.186'
            )
            with self.assertRaises(NotRT130FileError):
                Reftek130.from_file(test_file)

        with self.subTest('test_file_does_not_exist'):
            test_file = ''
            with self.assertRaises(FileNotFoundError):
                Reftek130.from_file(test_file)
